package com.foldor.soundboard;

import java.util.HashMap;
import java.util.Map;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.media.AudioManager;
import android.media.SoundPool;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.LinearLayout;
import android.widget.LinearLayout.LayoutParams;

public class SoundBoard extends Activity {
    private static final String TAG = "SoundBoard";
    
    SoundNameDatabase soundDB = new SoundNameDatabase(this);
    Map<Button, Sound> soundMap = new HashMap<Button, Sound>();
    
    SoundPool soundPool;
    AudioManager audioManager;
    
    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_soundboard);
        
        //soundDB.addSoundResources(); //Make sure the list of sounds is up-to-date
        
        LinearLayout soundBoard = (LinearLayout) findViewById(R.id.SoundBoardLayout);
        LinearLayout soundBoardRow = createRow();
        soundPool = new SoundPool(8, AudioManager.STREAM_MUSIC, 0);
        audioManager = (AudioManager) this.getSystemService(Context.AUDIO_SERVICE);
        
        //Get the number of buttons that are able to fit on each row.
        @SuppressWarnings("deprecation")
        int displayWidth = getWindowManager().getDefaultDisplay().getWidth();
        int usableWidth = displayWidth - (getDimensionSize(R.dimen.activity_horizontal_margin) * 2);
        int buttonWidth = getDimensionSize(R.dimen.buttonWidth) + (getDimensionSize(R.dimen.buttonMargin) * 2);
        int buttonsPerRow = (int) Math.floor(usableWidth / buttonWidth);
        
        Log.d(TAG, "buttonWidth: " + buttonWidth + " usableWidth: " + usableWidth);
        Log.d(TAG, "buttonsPerRow: " + buttonsPerRow);
        
        
        
        /**
         * Play the sound associated with the button when clicked.
         */
        OnClickListener soundButtonOnClickListener = new OnClickListener() {
            @Override
            public void onClick(View buttonView) {
                Button soundButton = (Button) buttonView;
                Sound sound = soundMap.get(soundButton);
                
                playSound(sound.soundID);
            }
        };
        
        
        /**
         * For each sound file, create a button and add it to the view.
         */
        for (Sound oneSound : soundDB.getAllSounds()) {
            if (oneSound.resource) {
                oneSound.soundID = soundPool.load(this, oneSound.soundResourceID, 1);
            }
            else {
                oneSound.soundID = soundPool.load(oneSound.soundPath, 1);
            }
            
            Log.d(TAG, "Name: " + oneSound.soundName + " soundId: " + oneSound.soundID);
                
            //Determine whether or not a new row needs to be created.
            int buttonsInRow = soundBoardRow.getChildCount();
            
            if (buttonsInRow >= buttonsPerRow) {
                soundBoard.addView(soundBoardRow);
                soundBoardRow = createRow();
            }
            
            //Create a button to the view for this sound, and add it to the view.
            Button soundButton = createSoundButton(oneSound.soundName, oneSound.soundID, soundButtonOnClickListener);
            soundBoardRow.addView(soundButton);
            
            //Pair the sound button with the sound object.
            soundMap.put(soundButton, oneSound);
        }
        
        //Add the final row to the soundBoard.
        soundBoard.addView(soundBoardRow);
        
        //Add one additional row for an edit button.
        LinearLayout newRow = createRow();
        Button editButton = new Button(this);
        
        editButton.setText("Edit SoundNames");
        editButton.setOnClickListener(new OnClickListener() {
            @Override
            public void onClick(View buttonView) {
                Intent intent = new Intent(getApplicationContext(), SoundBoardEdit.class);
                startActivity(intent);
            }
        });
        
        newRow.addView(editButton);
        soundBoard.addView(newRow);
    }
    
    /**
     * Get the number of pixels from a Dimension Resource
     * @param resource The resource ID for a dimension. ex. R.dimen.button_width
     * @return An integer of the number of real pixels.
     */
    protected int getDimensionSize(int resource) {
        return (int) Math.ceil(getResources().getDimension(resource));
    }
    
    /**
     * Create a new Horizontal LinearLayout and set the default parameters needed.
     * @return newRow
     */
    protected LinearLayout createRow() {
        LinearLayout newRow = new LinearLayout(this);
        
        int layoutHeight = getDimensionSize(R.dimen.layoutHeight);
        //int layoutVerticalMargin = getDimensionSize(R.dimen.activity_vertical_margin);
        //int layoutHorizontalMargin = getDimensionSize(R.dimen.activity_horizontal_margin);
        
        LayoutParams params = new LayoutParams(LayoutParams.MATCH_PARENT, layoutHeight);
        //params.setMargins(layoutHorizontalMargin, layoutVerticalMargin, 
        //                  layoutHorizontalMargin, layoutVerticalMargin);
        
        newRow.setLayoutParams(params);
        newRow.setOrientation(LinearLayout.HORIZONTAL);
        
        return newRow;
    }
    
    /**
     * Create a new button for the sound. 
     * This will create the button, add the text, and add the OnClickEvent to play the sound.
     * @param soundName The name of the sound to display on the button.
     * @param soundId The SoundPool ID
     * @param soundButtonOnClickListener the OnClickEvent to run when pressed.
     * @return The fully formatted button ready to put on the screen.
     */
    protected Button createSoundButton(String soundName, int soundId, OnClickListener soundButtonOnClickListener) {
        Button soundButton = new Button(this);
        
        soundButton.setId(soundId);
        soundButton.setText(soundName);
        soundButton.setOnClickListener(soundButtonOnClickListener);
        
        int buttonMargin = getDimensionSize(R.dimen.buttonMargin);
        int buttonWidth = getDimensionSize(R.dimen.buttonWidth);
        
        LayoutParams params = new LayoutParams(buttonWidth, LayoutParams.MATCH_PARENT);
        params.setMargins(buttonMargin, buttonMargin, buttonMargin, buttonMargin);
        
        soundButton.setLayoutParams(params);
        soundButton.setBackgroundColor(getResources().getColor(R.color.buttonBackground));
        soundButton.setTextColor(getResources().getColor(R.color.buttonTextColor));
        
        return soundButton;
    }
    
    /**
     * Play the MP3 associated with the passed Sound ID
     * @param soundID A Sound Pool ID to play.
     */
    protected void playSound(int soundID) {
        int streamVolume = audioManager.getStreamVolume(AudioManager.STREAM_MUSIC);
        soundPool.play(soundID, streamVolume, streamVolume, 1, 0, 1f);
    }
    
    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.main, menu);
        return true;
    }
    
}
