package com.foldor.soundboard;

import java.lang.reflect.Field;

import android.content.ContentValues;
import android.content.Context;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.Log;
import android.util.SparseArray;

/**
 *  Tracks the list of sounds to play on the SoundBoard. This works with resources in the RAW folder
 *  or files manually added from a folder on the device.
 */
public class SoundNameDatabase extends SQLiteOpenHelper {
    private static final String TAG = "SoundBoard";
    private static final int DATABASE_VERSION = 2;
    private static final String DATABASE_NAME = "Foldor_SoundBoard";
    private static final String SOUNDNAME_TABLE_NAME = "Sounds";
    private static final String SOUNDNAME_PATH = "Sound_Path";
    private static final String SOUNDNAME_RESOURCE_ID = "Resource_ID";
    private static final String SOUNDNAME_SOUND_NAME = "Sound_Name";
    private static final String SOUNDNAME_TABLE_CREATE =
                "CREATE TABLE IF NOT EXISTS " + SOUNDNAME_TABLE_NAME + " (" +
                        SOUNDNAME_PATH + " TEXT, " + 
                        SOUNDNAME_RESOURCE_ID + " INTEGER NOT NULL DEFAULT 0, " +
                        SOUNDNAME_SOUND_NAME + " TEXT);";
    
    SoundNameDatabase(Context context) {
        super(context, DATABASE_NAME, null, DATABASE_VERSION);
    }
    
    @Override
    public void onCreate(SQLiteDatabase db) {
        db.execSQL(SOUNDNAME_TABLE_CREATE);
        addSoundResources(db);
    }
    
    @Override
    public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
        Log.d(TAG, "Upgrading!");
        db.execSQL("DROP TABLE IF EXISTS " + SOUNDNAME_TABLE_NAME);
        db.execSQL(SOUNDNAME_TABLE_CREATE);
    }
    
    /**
     * Returns a list of content values to insert/update into the database. This will create one of a resource.
     * @param soundResourceID
     * @param soundName
     * @return soundValues
     */
    private ContentValues createContentValues(int soundResourceID, String soundName) {
        ContentValues soundValues = new ContentValues();
        
        soundValues.put(SOUNDNAME_RESOURCE_ID, soundResourceID);
        soundValues.put(SOUNDNAME_SOUND_NAME, soundName);
        
        return soundValues;
    }
    
    /**
     * Returns a list of content values to insert/update into the database. This will create one from a filepath.
     * @param soundPath
     * @param soundName
     * @return soundValues
     *
    private ContentValues createContentValues(String soundPath, String soundName) {
        ContentValues soundValues = new ContentValues();
        
        soundValues.put(SOUNDNAME_PATH, soundPath);
        soundValues.put(SOUNDNAME_SOUND_NAME, soundName);
        
        return soundValues;
    }
    
    /**
     * Force the resource sounds to be repopulated. 
     * This will overwrite any custom names, and bring back any deleted.
     */
    public void addSoundResources() {
        addSoundResources(this.getWritableDatabase());
    }
    
    /**
     * For each file in the raw folder, add it to the database with a default name from the file.
     * @param db
     */
    private void addSoundResources(SQLiteDatabase db) {
        //Get the list of MP3s in the raw folder.
        Field[] rawFields = R.raw.class.getFields();
        
        for (Field rawField : rawFields) {
            try {
                addSoundResource(rawField.getInt(null), rawField.getName(), db);                
            }
            catch (IllegalAccessException e) {
                //rawFields[i].getInt(null) has failed.
                Log.d(TAG, e.getMessage());
            }
            catch (Exception e) {
                //Catch all exceptions.
                e.printStackTrace();
            }
        }
    }
    
    /**
     * Add a single sound from the list of resources. Callable with the db object internally only.
     * @param soundResourceID
     * @param soundName
     * @param db
     */
    private void addSoundResource(int soundResourceID, String soundName, SQLiteDatabase db) {
        ContentValues soundValues = createContentValues(soundResourceID, soundName); 
        
        //Check if the resource exists in the DB already, and if not add it. If so, update it.
        db.insert(SOUNDNAME_TABLE_NAME, null, soundValues);
    }
    
    /**
     * Add a single sound from the list of resources.
     * @param sound_ResourceID
     * @param soundName
     */
    public void addSoundResource(int sound_ResourceID, String soundName) {
        addSoundResource(sound_ResourceID, soundName, this.getWritableDatabase());   
    }
    
    /**
     * Updates the name of a resource sound in the db
     * @param sound_ResourceID
     * @param soundName
     */
    private void updateSoundResource(int soundResourceID, String soundName) {
        SQLiteDatabase db = this.getWritableDatabase();
        ContentValues soundValues = createContentValues(soundResourceID, soundName);
        
        db.update(SOUNDNAME_TABLE_NAME, soundValues, SOUNDNAME_RESOURCE_ID + "=?", 
                new String[]{String.valueOf(soundResourceID)});
    }
    
    public void update(Sound sound) {
        if (sound.resource) {
            updateSoundResource(sound.soundResourceID, sound.soundName);
        }
    }
    
    /**
     * Deletes a resource sound from the DB.
     * @param sound_ResourceID
     */
    public void deleteSoundResource(int sound_ResourceID) {
        SQLiteDatabase db = this.getWritableDatabase();
        db.delete(SOUNDNAME_TABLE_NAME, SOUNDNAME_RESOURCE_ID + "=?",
                new String[] {String.valueOf(sound_ResourceID)});
        db.close();
    }
    
    /**
     * Returns the name of one single sound by it's resource ID.
     * @param sound_ResourceID
     * @return String Sound Name
     */
    public String getSoundResourceName(int sound_ResourceID) {
        SQLiteDatabase db = this.getReadableDatabase();
        
        Cursor sound = db.query(SOUNDNAME_TABLE_NAME, new String[] {SOUNDNAME_SOUND_NAME}, 
                SOUNDNAME_RESOURCE_ID + "=?", new String[] {String.valueOf(sound_ResourceID)}, 
                null, null, null);
        
        if (sound != null) {
            sound.moveToFirst();
            return sound.getString(0);
        }
        else {
            return "";
        }
    }
    
    /**
     * Returns a SpareArray<String> list of resource sounds with names in the database
     * @return SparseArray<String> soundMap
     */
    public SparseArray<String> getSoundResources() {
        SparseArray<String> soundMap = new SparseArray<String>();
        
        SQLiteDatabase db = this.getReadableDatabase();
        Cursor sounds = db.query(SOUNDNAME_TABLE_NAME, 
                new String[] {SOUNDNAME_RESOURCE_ID, SOUNDNAME_SOUND_NAME}, 
                null, null, null, null, null);
        
        if (sounds.moveToFirst()) {
            do {
                soundMap.put(sounds.getInt(0), sounds.getString(1));
            } while (sounds.moveToNext());
        }
        
        return soundMap;
    }
    
    /**
     * Return an array of Sound objects from the database.
     * @return Sound[] sounds
     */
    public Sound[] getAllSounds() {
        SQLiteDatabase db = this.getReadableDatabase();
        
        Cursor soundCursor = db.query(SOUNDNAME_TABLE_NAME, 
                new String[] {SOUNDNAME_SOUND_NAME, SOUNDNAME_PATH, SOUNDNAME_RESOURCE_ID}, 
                null, null, null, null, null);
        
        Sound[] sounds = new Sound[soundCursor.getCount()];
        
        if (soundCursor.moveToFirst()) {
            do {
                Sound sound = new Sound(soundCursor.getString(0), soundCursor.getString(1), soundCursor.getInt(2));
                sounds[soundCursor.getPosition()] = sound;
            } while (soundCursor.moveToNext());
        }
        
        return sounds;
    }
}
